package ch.dlmw.swisssignchallenge.controllers;

import ch.dlmw.swisssignchallenge.utils.JwtUtil;
import lombok.AllArgsConstructor;
import org.openapitools.api.TokenApi;
import org.openapitools.model.CreateSessionRequest;
import org.openapitools.model.CreateSessionResponse;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.web.bind.annotation.RestController;

@RestController
@AllArgsConstructor
public class TokenController implements TokenApi {
    private final AuthenticationManager authenticationManager;
    private final UserDetailsService userDetailsService;
    private final JwtUtil jwtUtil;

    @Override
    public ResponseEntity<CreateSessionResponse> createToken(CreateSessionRequest createSessionRequest) {
        authenticationManager.authenticate(
                new UsernamePasswordAuthenticationToken(createSessionRequest.getUsername(), createSessionRequest.getPassword())
        );

        UserDetails userDetails = userDetailsService.loadUserByUsername(createSessionRequest.getUsername());

        var response = new CreateSessionResponse(jwtUtil.generateToken(userDetails));

        return ResponseEntity.ok(response);
    }
}
