package ch.dlmw.swisssignchallenge.services.impl;

import ch.dlmw.swisssignchallenge.services.PdfService;
import lombok.AllArgsConstructor;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.stereotype.Service;

import java.net.MalformedURLException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.NoSuchElementException;

@Service
@AllArgsConstructor
public class PdfServiceImpl implements PdfService {
    private static final String FILE_DIRECTORY = "/srv/pdfs/";

    @Override
    public Resource getPdf(String name) {
        if (name.contains("..")) { // this is to avoid traversal attacks
            throw new IllegalArgumentException("Invalid file name: " + name);
        }

        Path filePath = Paths.get(FILE_DIRECTORY).resolve(name).normalize();
        Resource resource;
        try {
            resource = new UrlResource(filePath.toUri());
        } catch (MalformedURLException e) {
            throw new IllegalArgumentException("Invalid file name: " + name);
        }
        if (!resource.exists() || !resource.isReadable()) {
            throw new NoSuchElementException("Couldn't find file: " + name);
        }

        return resource;
    }
}
